%%   Differential conductance and IV curve
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @addtogroup Examples
%> @{
%> @file Differential_conductance.m
%> @brief Example to calculate the differential conductance and the IV curve through a junction made of a square lattice. Example taken from Eur. Phys. J. B 53, 537-549 (2006).
%> @param filenum The identification number of the filenema for the exported data (default is 1).
%> @reference
%> Eur. Phys. J. B 53, 537-549 (2006).
%> @Available
%> EQuUs v4.8 or later
%> @expectedresult
%> @image html Differential_conductance.jpg
%> @image latex Differential_conductance.jpg
%> @}
%
%> @brief Example to calculate the differential conductance and the IV curve through a junction made of a square lattice. Example taken from Eur. Phys. J. B 53, 537-549 (2006).
%> @param filenum The identification number of the filenema for the exported data (default is 1).
function Differential_conductance( filenum )


if ~exist('filenum', 'var')
    filenum = 1;
end

filename = mfilename('fullpath');
[directory, fncname] = fileparts( filename );

    % filename containing the input XML    
    inputXML = 'Basic_Input.xml';
    % Parsing the input file and creating data structures
    [Opt, param] = parseInput( fullfile( directory, inputXML ) );

    % filename containing the output XML
    outfilename = [fncname, '_',num2str( filenum )];
    % the output folder
    outputdir   = [];   
    % length of the junction
    height = 130;
    % width of the junction    
    width = 121;

    % Temperature in K
    T = 10;


    % The differential conductance as a function of the magnetiv field
    diff_conductance = [];
    % bias voltage at zero temperature
    bias_T0 = [];
    % The current at zero temperature
    current_T0 = [];
    % bias voltage at non-zero temperature
    bias = [];
    % The current at non-zero temperature
    current = [];
    
    % creating output directories
    setOutputDir();

    % Calculated the conductance
    CalculateTransport();

    % Creating the plots
    PlotFunction();

    % export the calculeted data
    save( [outputdir, filesep, outfilename,'.mat']);  

%% CalculateTransport
%> @brief Calculates the conductance
    function CalculateTransport()
        
        % number of energy points
        Edb = 300;
        % maximal bias
        bias_max = 0.025;
        
        island = structures('hole');
        island.center.x = width;
        island.center.y = height/2;
        island.radius = 30;
        
        % creating the Ribbon structure describing the junction
        cRibbon = Ribbon('width', width, 'height', height, 'EF', 1e-6, 'Opt', Opt, 'param', param, 'filenameOut', fullfile( outputdir, [outfilename, '.xml']) );
                
        % Setting the function handle for the circkle-shaped hard wall potential
        hScatterPotential = @(coordinates)(ScatterPotential(coordinates, island));
        
        % calculating the IV curve for zero temperature
        display('Calculating the IV curve for zero temperature.')
        cIV = IV( Opt, 'bias_max', bias_max, 'T', 0, 'junction', cRibbon, 'scatterPotential', hScatterPotential, 'gfininvfromHamiltonian', true, 'Edb', Edb);
		[current_T0, bias_T0, diff_conductance] = cIV.IVcalc('tuned_contact', 'right');
        
        % calculating the IV curve for nonzero temperature
        display('Calculating the IV curve for non-zero temperature.')
        cIV = IV( Opt, 'bias_max', bias_max, 'T', T, 'junction', cRibbon, 'scatterPotential', hScatterPotential, 'gfininvfromHamiltonian', true, 'Edb', Edb);
		[current, bias] = cIV.IVcalc('tuned_contact', 'right');  
                
    end
    



%% setOutputDir
%> @brief Sets output directory.
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = fullfile( directory, resultsdir );        
    end




%% PlotFunction
%> @brief Creates the plot
    function PlotFunction( )
        
        
        % creating figure in units of pixels
        figure1 = figure( 'Units', 'Pixels');
        
        % font size on the figure will be 16 points
        fontsize = 12;
        
        
        %************** conductance **********************
        % setting the limits of the axis
        x_lim = [0 25];
        y_lim = [0 16];
        
        % creating axes of the plot        
        axes_cond = axes('Parent',figure1, ...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...            
                'ylim', y_lim,...    
                'Box', 'on',...
                'Units', 'Pixels', ...
                'FontName','Times New Roman');
        hold on; 
        
        % plot the data
        plot(bias_T0*1000, real(diff_conductance), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);  
        
          
        
        % Create xlabel
        xlabel('E [meV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);

        % Create ylabel
        ylabel('G [2e^2/h]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        
        
        %************** IV curve **********************
        
        % creating axes of the plot        
        axes_current = axes('Parent',figure1, ...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...      
                'Box', 'on',...
                'Units', 'Pixels', ...
                'FontName','Times New Roman');
        hold on; 
        
        % plot the data
        plot(bias_T0*1000, real(current_T0)/max(bias_T0), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_current);  
        plot(bias*1000, real(current)/max(bias), 'Linewidth', 2, 'color', [1 0 0], 'Parent', axes_current);  
                
        % set the legends
        legend_labels = {'T=0 K', ['T= ', num2str(T), ' K'] };
        fig_legend = legend(axes_current, legend_labels);
        set(fig_legend, 'FontSize', fontsize*0.7, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthWest')
        
        
        % Create xlabel
        xlabel('U [meV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_current);

        % Create ylabel
        ylabel('I [2U_{0}e^2/h]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_current);


        
        % setting the position and margins of the plot, removing white
        % spaces for release dates greater than 2015
        ver = version('-release');
        if str2num(ver(1:4)) >= 2016
            % setting the position and margins of the plot, removing white spaces
            figure1.PaperPositionMode = 'auto';
            fig_pos = figure1.PaperPosition;
            figure1.PaperSize = [fig_pos(3) fig_pos(4)];                  
        end
        
        % set the figure position
        figure_pos = get( figure1, 'Position' );
        figure_pos(4) = figure_pos(4)/2;
        set( figure1, 'Position', figure_pos );
        
        % set the position of the conductance axis
        Position_cond = [0, 0, figure_pos(3)/2, figure_pos(4)];
        set(axes_cond, 'OuterPosition', Position_cond);    
        
        %set the position of the IV axis
        Position_current = [figure_pos(3)/2, 0, figure_pos(3)/2, figure_pos(4)];
        set(axes_current, 'OuterPosition', Position_current); 

        % export the figures
        print('-depsc2', fullfile(outputdir,[outfilename, '.eps']))
        print('-dpdf', fullfile(outputdir,[outfilename, '.pdf']))
        close(figure1)
        
        
    end



end
