%%    Eotvos Quantum Transport Utilities - XMLoutput
%    Copyright (C) 2009-2017 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    This function is the output filter to generate XML file from the
%    running parameters.
%%
function XMLoutput( filename, Opt, param )


descriptions = struct( ...
     'Decimation', 'Option for using Decimation. Type 1,2 or 3 to use it, 0 to not use it',  ...
     'Decimation_block', 'size of maximal block to decimate', ...
     'magnetic_field', 'Type 0 not to use magnetic field, or 1 to use magnetic field', ...
     'magnetic_field_trans_invariant', 'Set true (default) if a magnetic field is translational invariant along the scattering center, or false otherwise.', ...
     'Lattice_Type', 'Stirng containing the lattice type. See the documantation.', ...
    'Simple_Green_Function', 'Set 1 if a simple analytical surface Greens function computational method is about to use. (Only for square lattice without a magnetic field) ', ...
    'Linear_Regression_in_B', 'Set to 1 to use linear regression to calculate peierls integrals between the sites. Useful when dealing with homogenius magnetic field', ....
    'BdG', 'Set 1 to use the Bogoliubov de Gennes model, or 0 for normal system. ', ...
    'Spin', 'Set 1 to include electron spin in the computations, or 0 otherwise. ', ...
    'Silent', 'Set to 1 in order to supress output messages',  ...
    'usingDualModes', 'Set 1 to use dual modes in the calculations, or 0 to use the left and right sided eigenvectors instead. ', ...
    'debug', 'Set 1 to export debug informations into the debug.txt file, or 0 otherwise.', ...
    'workers', 'Number of the workers in the parallel for loops', ...
    'custom_Hamiltonians', 'The identifier of the external source of Hamiltonians. See the documentation for details.', ...
    'computing_parameters', 'Parameters used in calculations', ...
    'Scatter_region', 'Parameters used to create scattering region', ...
     'epsilon',  'On-site energy', ...
     'vargamma', 'Hopping amplitude', ...
     'vargamma2', 'Hopping amplitude', ...
     'vargamma3', 'Hopping amplitude', ...
     'pair_potential', 'Superconducting pair potential, with abs standing for the absolute value and phase for the phase of the complex number', ...
     'End_Type', 'Lead end type in case of hexagonal lattice', ...
     'Lead_parameters', 'Physical parameters of the individual leads.', ...
     'NofLeads', 'Number of leads', ...
     'vargamma_sc', 'Alternative hopping to the scattering reigon' , ...
     'orientation',  'Orientation of the lead', ...
     'lead', 'Physical parameters of a lead.' , ...
     'shape', 'Dimensions of the scattering region', ...
     'width', 'Width of the scattering center', ...
     'height', 'length (height) of the scattering center' ,...
     'M', 'Number of sites in the cross section of the lead', ...
     'AtomNode', 'Description of an atomic site',...
     'PL', 'Index of the principal layer',...
     'Id', 'Identification number of the site',...
     'Lead_Orientation', 'Orientation of the lead.' ...
     );

cCommonFunctions = CommonFunctions();

XMLtree = [];
root       = [];

setRootElement();
setComputingParameters();
setScatteringRegion();
setLeads();

cCommonFunctions.xmlwrite( filename, XMLtree );



%% SetRoot element
    function setRootElement()

        if cCommonFunctions.isOctave()
            % Create a Xerces DOM document object 
            XMLtree = javaObject ('org.apache.xerces.dom.DocumentImpl');
            % Append a root node to the document 
            XMLtree.appendChild (XMLtree.createElement ('parameters'));
        else

            XMLtree = com.mathworks.xml.XMLUtils.createDocument('parameters');
        end
        root = XMLtree.getDocumentElement;
        root.setAttribute('identifier', cCommonFunctions.getProgramShortName() );
        root.setAttribute('version', num2str( cCommonFunctions.getVersion() ) );
        
    end


%% SetComputingParameters
    function setComputingParameters()
        computing_parametersNode = XMLtree.createElement('computing_parameters');
        computing_parametersNode.setAttribute('description', descriptions.computing_parameters);
       
        
        fieldnamesOpt = fieldnames( Opt );
        for idx = 1:length(fieldnamesOpt)
            fieldname = fieldnamesOpt{idx};
            if isempty( Opt.(fieldname) )
                continue
            end
            
            XMLnode = createXMLnode( fieldname, Opt );
            computing_parametersNode.appendChild(XMLnode);       
            
        end
        
        root.appendChild(computing_parametersNode); 
        
    end

%% setScatteringRegion()
    function setScatteringRegion()
        if isempty( param.scatter)
            return
        end
        
        nodename = 'Scatter_region';
        Scatter_regionNode = XMLtree.createElement( nodename );
        Scatter_regionNode.setAttribute('description', descriptions.(nodename));
        
        fieldnamesScatter = fieldnames( param.scatter );
        for idx = 1:length( fieldnamesScatter)
            fieldname = fieldnamesScatter{idx};
            if isempty( param.scatter.(fieldname) )
                continue
            end
            
            if strcmp( fieldname, 'shape' )
                shapeNodeName = 'shape';
                ShapeNode = XMLtree.createElement( shapeNodeName );
                ShapeNode.setAttribute('description', descriptions.(shapeNodeName));
                
                if isempty( param.scatter.shape )
                    continue
                elseif isempty( param.scatter.shape.width )
                    continue
                elseif isempty( param.scatter.shape.height )
                    continue                    
                end
                
                fieldnamesShape = fieldnames( param.scatter.shape );
                for jdx = 1:length( fieldnamesShape)
                    fieldnameShape = fieldnamesShape{jdx};
                    XMLnode = createXMLnode( fieldnameShape, param.scatter.shape );
                    ShapeNode.appendChild(XMLnode);          
                end
                
                Scatter_regionNode.appendChild(ShapeNode);    
            elseif strcmp( fieldname, 'Atom' )
                if isempty( param.scatter.Atom )
                    continue
                end
                appendAtomNodes( param.scatter.Atom, Scatter_regionNode )               
            else
                XMLnode = createXMLnode( fieldname, param.scatter );
                Scatter_regionNode.appendChild(XMLnode);                   
            end
            
   
        end
        
        root.appendChild(Scatter_regionNode); 
    end
%% setLeads();
    function setLeads()
        if isempty( param.Leads)
            return
        end
        
        nodename = 'Lead_parameters';
        LeadsNode = XMLtree.createElement( nodename );
        LeadsNode.setAttribute('description', descriptions.(nodename));
        
        XMLnode = XMLtree.createElement( 'NofLeads' );
        XMLnode.setAttribute('value', num2str(length(param.Leads)));
        XMLnode.setAttribute('description', descriptions.('NofLeads'));
        LeadsNode.appendChild(XMLnode); 
        
        for idx = 1:length(param.Leads)
            setLead( idx )
        end
        
        root.appendChild(LeadsNode); 
        
        % set individual leads
        function setLead( leadnum  )
            leadnodename = 'lead';
            LeadNode = XMLtree.createElement( leadnodename );
            LeadNode.setAttribute('num', num2str(leadnum));
        
            fieldnamesLead = fieldnames( param.Leads{leadnum} );
            for jdx = 1:length( fieldnamesLead)
                fieldname = fieldnamesLead{jdx};
                if isempty( param.Leads{leadnum}.(fieldname) )
                    continue
                end
                
                if strcmp( fieldname, 'Atom' )
                    if isempty( param.scatter.Atom )
                        continue
                    end
                    appendAtomNodes( param.Leads{leadnum}.Atom, LeadNode );
                else
            
                    XMLnode = createXMLnode( fieldname, param.Leads{leadnum} );
                    LeadNode.appendChild(XMLnode); 
                end
            end
            
            LeadsNode.appendChild(LeadNode); 
            
            
        end
        
        
    
    end




%% appendAtomNodes
    function appendAtomNodes( structure, parentXMLnode )

        AtomNodeName = 'Atom';
        numAtoms = length( structure );           
        for jdx = 1:numAtoms
            AtomNode = XMLtree.createElement( AtomNodeName );
            AtomNode.setAttribute('description', descriptions.AtomNode);
                    
            if isempty( structure )
                continue
            end
                
            fieldnamesAtom = fieldnames( structure(jdx) );
                for kdx = 1:length( fieldnamesAtom)
                    fieldnameAtom = fieldnamesAtom{kdx};
                    XMLnode = createXMLnode( fieldnameAtom, structure(jdx) );
                    XMLnode.setAttribute('description', descriptions.(fieldnameAtom));
                    AtomNode.appendChild(XMLnode);          
                end    
                parentXMLnode.appendChild(AtomNode);    
        end
        
    end



%% createXMLNode
    function XMLnode = createXMLnode( fieldname, structure )
            XMLnode = XMLtree.createElement( fieldname );
            if ischar( structure.(fieldname) )
                paramval = structure.(fieldname);
                XMLnode.setAttribute('value', paramval);
            elseif isscalar( structure.(fieldname) ) && isreal(structure.(fieldname))
                paramval = num2str( structure.(fieldname) );
                XMLnode.setAttribute('value', paramval);
            elseif isscalar( structure.(fieldname) ) && ~isreal(structure.(fieldname))
                absval = num2str( abs(structure.(fieldname) ));
                angleval = num2str( angle(structure.(fieldname) ));
                XMLnode.setAttribute('abs', absval);
                XMLnode.setAttribute('phase', angleval);
            else
                return
            end
            
            
            if isfield( descriptions, fieldname )
                XMLnode.setAttribute('description', descriptions.(fieldname));
            end       
        
    end


end
