!======================================================================
!     Wrapper module to support the conversion of Fortran types into C compatible types.
!     This file is part of the EQuUs package. (for details see: http://eqt.elte.hu/equus/home)
!     Copyright (C) 2016 Peter Rakyta, Ph.D.
!
!     This program is free software: you can redistribute it and/or modify
!     it under the terms of the GNU General Public License as published by
!     the Free Software Foundation, either version 3 of the License, or
!     (at your option) any later version.
! 
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with this program.  If not, see http://www.gnu.org/licenses/.
!     
!======================================================================
MODULE EQuUs_C

  use iso_c_binding


integer, parameter, public :: dp = selected_real_kind(14,100)
integer, parameter, public :: sp = selected_real_kind(6,30)


! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief An EQuUs element prototype   
type, public :: EQelement
! Number of dimensions of the element
  TYPE(C_PTR) :: dim
! Array of dimensions 
  TYPE(C_PTR) :: dims
! C type pointer to the value of the element
  TYPE(C_PTR) :: value
! MATLAB class id of the value (see http://www.mathworks.com/help/matlab/apiref/mxclassid.html)
  TYPE(C_PTR) :: classID
! String containing the variable name
  TYPE(C_PTR) :: elementname
end type

! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief An EQuUs structure prototype 
type, public :: EQstruct
! C type pointer pointing to the array of EQuUs element prototypes
  TYPE(C_PTR) :: fields
! Number of the stored EQuUs element prototypes
  integer :: numfields
end type


interface assocElement
  module procedure assocIntElement
  module procedure assocRealElement
  module procedure assocIntVectorElement
  module procedure assocIntMatrixElement
  module procedure assocRealVectorElement
  module procedure assocRealMatrixElement
  module procedure assocLogicalElement
  module procedure assocStringElement
  module procedure assocStringElement2
  module procedure assocStructElement
end interface

interface allocateElement
  module procedure allocateElementArray
  module procedure allocateElementSingle
end interface


!****************************************************************      
  CONTAINS
!****************************************************************


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Add an EQelement to an array of EQelements
!! 
!> @param eq_elements An array of EQelements
!> @param[in] eq_elements A pointer to an EqElement instance
  subroutine addElement( eq_elements, eq_element )
    TYPE(EQelement), pointer :: eq_element
    TYPE(EQelement), pointer :: eq_elements(:)
    TYPE(EQelement), pointer :: eq_elements_new(:)
    integer elementNum



    if (.not.ASSOCIATED(eq_elements)) then
      allocate(eq_elements(1))
      eq_elements(1) = eq_element
      return
    end if

    elementNum = SIZE( eq_elements, 1)
    allocate( eq_elements_new(elementNum+1) )

    do idx = 1, elementNum
      eq_elements_new(idx) = eq_elements(idx)
    end do

    eq_elements_new(elementNum+1) = eq_element

    deallocate( eq_elements )
    allocate( eq_elements(elementNum+1) )
    do idx = 1, elementNum+1
      eq_elements(idx) = eq_elements_new(idx)
    end do

    deallocate( eq_elements_new )
    
  end subroutine addElement


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Allocates a memory for an array of EQelement 
!! 
!> @param[out] eq_elements Pointer to an array of of EQelements
!> @param[in] num The length of the array to be allocated
  subroutine allocateElementArray( eq_elements, num )
    TYPE(EQelement), pointer :: eq_elements(:)
    integer :: num
    integer :: idx

    allocate( eq_elements(num) )

    do idx = 1, num
      eq_elements(idx)%elementname = C_LOC( NULL() )
      eq_elements(idx)%dim = C_LOC( NULL() )
      eq_elements(idx)%dims = C_LOC( NULL() )
      eq_elements(idx)%value = C_LOC( NULL() )
      eq_elements(idx)%classID = C_LOC( NULL() )
    end do

  end subroutine allocateElementArray


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Allocates a memory for an EQelement 
!! 
!> @param[out] eq_element An instance of EQelement 
  subroutine allocateElementSingle( eq_element )
    TYPE(EQelement), pointer :: eq_element

    allocate( eq_element )

    eq_element%elementname = C_LOC( NULL() )
    eq_element%dim = C_LOC( NULL() )
    eq_element%dims = C_LOC( NULL() )
    eq_element%value = C_LOC( NULL() )
    eq_element%classID = C_LOC( NULL() )

  end subroutine allocateElementSingle

!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Deallocates the memory allocated in an EQelement 
!! 
!> @param[in] eq_element An instance of EQelement 
  subroutine deallocateElement( eq_element )
    TYPE(EQelement) :: eq_element

    BYTE, pointer :: ptr => NULL()

    call c_f_pointer( eq_element%elementname, ptr )
    if (ASSOCIATED(ptr)) then
      deallocate( ptr )
      nullify (ptr )
    end if

    call c_f_pointer( eq_element%dim, ptr )
    if (ASSOCIATED(ptr)) then
      deallocate( ptr )
      nullify (ptr )
    end if

    call c_f_pointer( eq_element%dims, ptr )
    if (ASSOCIATED(ptr)) then
      deallocate( ptr )
      nullify (ptr )
    end if

    call c_f_pointer( eq_element%value, ptr )
    if (ASSOCIATED(ptr)) then
      deallocate( ptr )
      nullify (ptr )
    end if

    call c_f_pointer( eq_element%classID, ptr )
    if (ASSOCIATED(ptr)) then
      deallocate( ptr )
      nullify (ptr )
    end if

  end subroutine deallocateElement


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates an EQstruct element to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocStructElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    TYPE(EQstruct) :: value
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)

    allocate( dims(1), stat=iostat)
    dims(1) = 1

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    classID(1) = 2


    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(value)
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocStructElement

!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran string to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The value related to the Fortran type.  
  subroutine assocStringElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    character(len=*), pointer :: value(:)
    TYPE(C_PTR), pointer :: values(:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)
    integer idx

    allocate( dims(1), stat=iostat)
    dims(1) = SIZE( value,1)

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    classID(1) = 4

    allocate(values( dims(1) ))
    do idx = 1, dims(1) 
      value(idx) = trim(value(idx))//C_NULL_CHAR
      values(idx) = C_LOC( value(idx) )
    end do

    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC( values(1) )
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocStringElement



!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran string to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The value related to the Fortran type.  
  subroutine assocStringElement2( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    character(len=*) :: value
    character(len(trim(value))+1), pointer :: value_tmp
    TYPE(C_PTR), pointer :: values(:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)
    integer idx

    allocate( dims(1), stat=iostat)
    dims(1) = 1

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    classID(1) = 4

    allocate(values(1))
    allocate(value_tmp)
    value_tmp = trim(value)//C_NULL_CHAR
    values(1) = C_LOC(value_tmp)

    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC( values(1) )
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocStringElement2


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran logical to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocLogicalElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    logical :: value
    integer, pointer :: pvalue(:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)

    allocate( dims(1), stat=iostat)
    dims(1) = 1

    allocate( pvalue(1), stat=iostat)
    if ( value.eqv..true.)  then
      pvalue(1) = 1
    else
      pvalue(1) = 0
    endif

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    classID(1) = 3

    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(pvalue(1))
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocLogicalElement


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran integer to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocIntElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    integer :: value
    integer, pointer :: pvalue(:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)


    allocate( dims(1), stat=iostat)
    dims(1) = 1

    allocate(pvalue(1), stat=iostat)
    pvalue(1) = value

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    classID(1) = 12

    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(pvalue(1))
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocIntElement


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran real(dp) to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocRealElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    real(dp) :: value
    real(dp), pointer :: pvalue
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)

    allocate( dims(1), stat=iostat)
    dims(1) = 1

    allocate(pvalue, stat=iostat)
    pvalue = value

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    if ( dp.EQ.8) then
      classID(1) = 6
    else
      print *, 'Undetermined real type !!!!!!!!!!!'
      classID(1) = 0
    end if

    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(pvalue)
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocRealElement



!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran integer array to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocIntVectorElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    integer, pointer :: value(:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)


    allocate( dims(1), stat=iostat)
    dims(1) = SIZE(value, 1)

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    classID(1) = 12


    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(value(1))
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocIntVectorElement


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran integer matrix to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocIntMatrixElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    integer, pointer :: value(:,:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)

    allocate( dims(2), stat=iostat)
    dims(1) = SIZE(value, 1)
    dims(2) = SIZE(value, 2)

    allocate( dim(1), stat=iostat)
    dim(1) = 2
    allocate( classID(1), stat=iostat)
    classID(1) = 12

    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(value(1,1))
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocIntMatrixElement


!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran real(dp) array to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocRealVectorElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    real(dp), pointer :: value(:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)

    allocate( dims(1), stat=iostat)
    dims(1) = SIZE(value, 1)

    allocate( dim(1), stat=iostat)
    dim(1) = 1
    allocate( classID(1), stat=iostat)
    if ( dp.EQ.8) then
      classID(1) = 6
    else
      print *, 'Undetermined real type !!!!!!!!!!!'
      classID(1) = 0
    end if


    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(value(1))
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocRealVectorElement



!-------------------------------------------------------------------------------------------------
! DESCRIPTION:
!> @file EQuUs_C.f90
!> @brief Associates fortran real(dp) matrix to an EQuUs element prototype 
!! 
!> @param[out] eq_element An instance of EQelement 
!> @param[in] elementname The name of the element
!> @param[in] value The valu related to the Fortran type.  
  subroutine assocRealMatrixElement( eq_element, elementname, value )
  
    TYPE(EQelement) :: eq_element
    character(len=*) :: elementname
    character(len(elementname)+1), pointer :: pelementname
    real(dp), pointer :: value(:,:)
    integer, pointer :: dims(:)
    integer, pointer :: dim(:)
    integer, pointer :: classID(:)

    allocate( dims(2), stat=iostat)

    dims(1) = SIZE(value, 1)
    dims(2) = SIZE(value, 2)

    allocate( dim(1), stat=iostat)
    dim(1) = 2
    allocate( classID(1), stat=iostat)
    if ( dp.EQ.8) then
      classID(1) = 6
    else
      print *, 'Undetermined real type !!!!!!!!!!!'
      classID(1) = 0
    end if

    allocate(pelementname)
    pelementname = trim(elementname)//C_NULL_CHAR

    eq_element%elementname = C_LOC(pelementname)
    eq_element%dim = C_LOC(dim(1))
    eq_element%dims = C_LOC( dims(1) )
    eq_element%value = C_LOC(value(1,1))
    eq_element%classID = C_LOC(classID(1))

  end subroutine assocRealMatrixElement






end module
