%%    Eotvos Quantum Transport Utilities - Coordinates
%    Copyright (C) 2018 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @addtogroup structures Structures
%> @{
%> @file Coordinates.m
%> @brief Structure containing the coordinates and other quantum number identifiers of the sites in the Hamiltonians.
%> @} 
%> @brief Structure containing the coordinates and other quantum number identifiers of the sites in the Hamiltonians.
%%
classdef Coordinates

    properties
        %> A vector containing the $x$ coordinates of the sites in units of the lattice constant. 
        x
        %> A vector containing the $y$ coordinates of the sites in units of the lattice constant. 
        y 
        %> A vector containing the $z$ coordinates of the sites in units of the lattice constant. 
        z
        %> The lattice vector of the translational invariant lead in units of the lattice constant. 
        a
        %> The lattice vector in the transverse direction in units of the lattice constant. 
        b
        %> Units of length
        LatticeConstant
        %> Cell array containing the orbital types of the sites
        OrbitalTypes
        %> Array of logicals. Used in system including a spin degree of freedom. Values "true" gives sites with spin up, and values "false" stand for the spin down sites. 
        spinup
        %> Array of logicals. Used in the Bogoliubov de Gennes Hamiltonians. Values "true" gives sites corresponding to the "u" component of the wave function, and values "false" stand for the sites of the "v" component of the wave function. 
        BdG_u
    end



methods (Access=public)
    
%% Contructor of the class
%> @brief Constructor of the class.
%> @return An instance of the class
    function obj = Coordinates() 
        
        % initializing class members
        obj.x = [];
        obj.y = [];
        obj.z = [];
        obj.a = [];
        obj.b = [];
        obj.LatticeConstant = 1;
        obj.OrbitalTypes = [];
        obj.spinup = [];
        obj.BdG_u = [];
        
    end
    
%% RemoveSites
%> @brief Removes sites from the structure
%> @param indexes Logical array. Sites with true values will be removed from the structure.
%> @return Returns with the modified structure
    function ret = RemoveSites( obj, indexes )    
        
        % do a for loop for class properties
        meta_data = metaclass(obj);
        
        for idx = 1:length(meta_data.PropertyList)
            prop_name = meta_data.PropertyList(idx).Name;
            if strcmpi(prop_name, 'a') || strcmpi(prop_name, 'b') || strcmpi(prop_name, 'LatticeConstant')
                continue
            end
            
            if ~isempty( obj.(prop_name) )
                if iscell( obj.(prop_name) )
                    obj.(prop_name) = obj.(prop_name){~indexes};
                else
                    obj.(prop_name) = obj.(prop_name)(~indexes);
                end
            end
        end    
        
        ret = obj;
        
    end
    
    
%% KeepSites
%> @brief Keep only sites in the structure defined by the input parameter #indexes
%> @param indexes Logical array. Sites with true values will be kept in the structure.
%> @return Returns with the modified structure
    function ret = KeepSites( obj, indexes )   
        
        % do a for loop for class properties
        meta_data = metaclass(obj);
        
        for idx = 1:length(meta_data.PropertyList)
            prop_name = meta_data.PropertyList(idx).Name;
            if strcmpi(prop_name, 'a') || strcmpi(prop_name, 'b') || strcmpi(prop_name, 'LatticeConstant')
                continue
            end
            
            if ~isempty( obj.(prop_name) )
                if iscell( obj.(prop_name) )
                    obj.(prop_name) = obj.(prop_name){indexes};
                else
                    obj.(prop_name) = obj.(prop_name)(indexes);
                end
            end
        end  
        
        ret = obj;
        
    end   
    
    
%% Combine
%> @brief Combines the present structure with the instance given as an input
%> @param cCoordinates An instance of a structure to be combined with.
%> @return Returns with the combinied structure
    function ret = Combine( obj, cCoordinates )    
        
        % check the type of the input
        if ~strcmp( class( cCoordinates ), class(obj) ) 
            error(['EQuUs:', class(obj), 'Combine'], 'The input class has a wrong type');
        end
        
        % creating instance to be returned
        ret = structures('coordinates');
        
        % do a for loop for class properties
        meta_data = metaclass(obj);
        
        for idx = 1:length(meta_data.PropertyList)
            prop_name = meta_data.PropertyList(idx).Name;
            if strcmpi(prop_name, 'a') || strcmpi(prop_name, 'b') || strcmpi(prop_name, 'LatticeConstant')
                
                % checking whether the structure to be combined with describes the same strucutre 
                if norm( obj.(prop_name) - cCoordinates.(prop_name) ) > 0
                    error(['EQuUs:', class(obj), 'Combine'], 'The structure to be combined with is describing a different structure.');
                end
                
                ret.(prop_name) = obj.(prop_name);
                continue
            end
            
            % combining data arrays
            if ~isempty( obj.(prop_name) ) && ~isempty( cCoordinates.(prop_name) )
                ret.(prop_name) = [obj.(prop_name); cCoordinates.(prop_name)];
            elseif ~isempty( obj.(prop_name) ) && isempty( cCoordinates.(prop_name) )
                error(['EQuUs:', class(obj), 'Combine'], ['Data array ', prop_name, ' is empty in the input structure']);
            elseif isempty( obj.(prop_name) ) && ~isempty( cCoordinates.(prop_name) )
                error(['EQuUs:', class(obj), 'Combine'], ['Data array ', prop_name, ' is empty in the initial structure']);
            end
        end    
        
    end    


%% Combine
%> @brief Combines the present structure with the instance given as an input
%> @param shift_vector A two- or three component vector describing the coordinate shift.
%> @return Returns with the shifted structure
    function ret = Shift( obj, shift_vector )   

        ret = obj;
        
        % shifting the x and y coordinates
        ret.x = obj.x + shift_vector(1);
        ret.y = obj.y + shift_vector(2);

        % shifting the z coordinates
        if ~isempty( obj.z ) && length(shift_vector)>2
            ret.z = obj.z + shift_vector(3);
        end
        
    end    
    
%% Transform2BdG
%> @brief Transforms sthe structure into the BdG model in the Nambu space representation.
%> @return Returns with the transformed instance of the structure
    function ret = Transform2BdG( obj )
        
        numel = [];
        
        fnames = fieldnames( obj );
        for idx = 1:length(fnames)
            fname = fnames{idx};
            if strcmp(fname, 'a') || strcmp(fname, 'b') || strcmp(fname, 'LatticeConstant')
                continue
            end                    
            
            if ~isempty( obj.(fname) )
                numel = length( obj.(fname) );
            end
            
            obj.(fname) = [ obj.(fname); obj.(fname) ];
        end
           
        if ~isempty(numel)
            obj.BdG_u = [ true(numel,1); false(numel,1)];
        end
        
        ret = obj;
        
    end    
    
    
end % public methods end

end