%%    Eotvos Quantum Transport Utilities - UtilsBase
%    Copyright (C) 2009-2017 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%> @addtogroup utilities Utilities
%> @{
%> @file UtilsBase.m
%> @brief This class is a base class containing common properties and methods utilized in several other classes.
%> @} 
%> @brief This class is a base class containing common properties and methods utilized in several other classes.
%> @Available
%> EQuUs v4.9 or later
%%
classdef UtilsBase < CommonFunctions & Messages & FermiDirac
    
    properties ( Access = public )
        %> An instance of class #NTerminal (or its subclass) representing the junction
        junction
        %>An instance of structure #BandWidth
        BandWidth
        %>logical value. Set true (default) for solve the Dyson equation with the self energies, or false otherwise.
        useSelfEnergy
        %>Function handle pot = f(#coordinates) for the additional potential to be applied in the scattering region
        scatterPotential   
        %>logical value. Set true for calculate the Greens function of the scattering region from Hamiltonian, or false (default) otherwise.
        gfininvfromHamiltonian        
		%> cell array of optional parameters. (For details see #InputParsing)
        varargin
    end    
    
    
    methods (Access=public)
    
%% Contructor of the class
%> @brief Constructor of the class.
%> @param Opt an instance of class #Opt.
%> @param varargin Cell array of optional parameters. For details see #InputParsing.
%> @return An instance of the class
    function obj = UtilsBase( Opt, varargin )   
        
        obj = obj@Messages( Opt );
        
        obj.junction           = [];
        obj.BandWidth          = [];
        obj.useSelfEnergy      = [];
        obj.scatterPotential   = [];
        obj.gfininvfromHamiltonian = [];

        obj.varargin = varargin;        
        
        

        if strcmpi(class(obj), 'UtilsBase')             
            obj.InputParsing( varargin{:} );
        end
   
        
        
    end
    
    
%% setEnergy
%> @brief Sets the energy for the calculations
%> @param newE The value of the energy in the same units as the Hamiltonian.
%> @param varargin Cell array of optional parameters (https://www.mathworks.com/help/matlab/ref/varargin.html):
%> @param 'junction' An instance of class #NTerminal (or its derived class) representing the junction.
    function SetEnergy( obj, newE, varargin ) 
        p = inputParser;
        p.addParameter('junction', obj.junction);
        p.parse(varargin{:});
        
        junction_loc = p.Results.junction;
        junction_loc.setEnergy( newE ) 
    end        
    
  
    
    end % public methods
    
    
    methods (Access=protected)    
        
        
%% InputParsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Cell array of optional parameters (https://www.mathworks.com/help/matlab/ref/varargin.html):
%> @param 'T' The temperature in Kelvin (scalar or an array) 
%> @param 'mu' The Chemical potential in the same unit as other energy scales in the Hamiltonians.
%> @param 'scatterPotential' A function handle pot=f( #coordinates ) or pot=f( #CreateHamiltonians, Energy) for the potential to be applied in the Hamiltonian (used when FiniteGreensFunctionFromHamiltonian=true).
%> @param 'gfininvfromHamiltonian' Logical value. Set true calculate the surface Greens function of the scattering region from the Hamiltonaian of the scattering region, or false (default) to calculate it by the fast way (see Phys. Rev. B 90, 125428 (2014) for details).
%> @param 'useSelfEnergy' Logical value. Set true to use the self energies of the leads in the Dyson equation, or false (default) to use the surface Green function instead.
%> @param 'junction' An instance of class #NTerminal (or its derived class) representing the junction.
    function InputParsing( obj, varargin )
        
        p = inputParser;
        p.addParameter('T', obj.T);
        p.addParameter('mu', 0, @isscalar);
        p.addParameter('scatterPotential', []);
        p.addParameter('gfininvfromHamiltonian', false);
        p.addParameter('useSelfEnergy', true);
        p.addParameter('junction', []);     
        
        p.parse(varargin{:});
        
        InputParsing@FermiDirac( obj, 'T', p.Results.T, ...
                                'mu', p.Results.mu);

        obj.scatterPotential    = p.Results.scatterPotential;
        obj.junction            = p.Results.junction;
        obj.gfininvfromHamiltonian  = p.Results.gfininvfromHamiltonian;
        obj.useSelfEnergy        = p.Results.useSelfEnergy;
                
        
        if ~strcmpi( class(obj.junction), 'NTerminal' )        
            supClasses = superclasses(obj.junction);
            if sum( strcmp( supClasses, 'NTerminal' ) ) == 0
                error(['EQuUs:Utils:', class(obj), ':InputParsing'], 'Element junction is not valid.');
            end
        end
               

    end % 
    
    
end % protected methods        
    
end